from django.urls import path, include, re_path
from django.contrib import admin
from rest_framework.routers import DefaultRouter
from rest_framework import permissions
from drf_yasg.views import get_schema_view
from drf_yasg import openapi
from .views import (
    GameViewSet,
    GuildViewSet,
    PlayerViewSet,
    hello_world,
    PlayerGameDataViewSet,
    GuildGameDataSchemaViewSet,
    PlayerGuildViewSet,
    WowClassesViewSet,
    LoginView,
)

router = DefaultRouter()
router.register(r"games", GameViewSet)
router.register(r"guilds", GuildViewSet)
router.register(r"players", PlayerViewSet)
router.register(r"playergamedata", PlayerGameDataViewSet)
router.register(r"guildgamedataschema", GuildGameDataSchemaViewSet)
router.register(r"playerguild", PlayerGuildViewSet)
router.register(r"wowclasses", WowClassesViewSet)

schema_view = get_schema_view(
    openapi.Info(
        title="Guild Data API",
        default_version="v1",
        description="API for Guild Data",
        terms_of_service="https://www.google.com/policies/terms/",
        contact=openapi.Contact(email="yourmom@lole.com"),
        license=openapi.License(name="License"),
    ),
    public=True,
    permission_classes=(permissions.AllowAny,),
)


urlpatterns = [
    path("", hello_world, name="index"),
    path("admin/", admin.site.urls),
    path("api/", include(router.urls)),
    path("login/", LoginView.as_view(), name="login"),
    # Swagger
    re_path(
        r"^swagger(?P<format>\.json|\.yaml)$",
        schema_view.without_ui(cache_timeout=0),
        name="schema-json",
    ),
    re_path(
        r"^swagger/$",
        schema_view.with_ui("swagger", cache_timeout=0),
        name="schema-swagger-ui",
    ),
]
