from django.http import HttpResponse
from rest_framework import viewsets, status
from django.contrib.auth import authenticate, login
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.permissions import AllowAny, IsAuthenticated, IsAdminUser
from drf_yasg.utils import swagger_auto_schema
from drf_yasg import openapi
from .models import (
    Game,
    Guild,
    Player,
    PlayerGuild,
    PlayerGameData,
    GuildGameDataSchema,
    WowClasses,
)
from .serializers import (
    GameSerializer,
    GuildSerializer,
    PlayerSerializer,
    PlayerGuildSerializer,
    PlayerGameDataSerializer,
    GuildGameDataSchemaSerializer,
    WowClassesSerializer,
    LoginSerializer,
)


class LoginView(APIView):
    permission_classes = [AllowAny]
    @swagger_auto_schema(
        operation_description="Login to the system",
        request_body=LoginSerializer,
        responses={
            status.HTTP_200_OK: openapi.Response('Login Successful'),
            status.HTTP_401_UNAUTHORIZED: openapi.Response('Invalid Credentials'),
        }
    )
    def post(self, request, format=None):
        serializer = LoginSerializer(data=request.data)
        if serializer.is_valid():
            username = serializer.validated_data['username']
            password = serializer.validated_data['password']
            user = authenticate(request, username=username, password=password)
            if user is not None:
                login(request, user)
                return Response({'detail': 'Login successful'}, status=status.HTTP_200_OK)
            else:
                return Response({'detail': 'Invalid Credentials'}, status=status.HTTP_401_UNAUTHORIZED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


def hello_world(request):
    return HttpResponse("Hello, World!")


class GameViewSet(viewsets.ModelViewSet):
    permission_classes = [IsAuthenticated]
    queryset = Game.objects.all()
    serializer_class = GameSerializer


class GuildViewSet(viewsets.ModelViewSet):
    permission_classes = [IsAuthenticated]
    queryset = Guild.objects.all()
    serializer_class = GuildSerializer


class PlayerViewSet(viewsets.ModelViewSet):
    permission_classes = [IsAuthenticated]
    queryset = Player.objects.all()
    serializer_class = PlayerSerializer


class PlayerGameDataViewSet(viewsets.ModelViewSet):
    permission_classes = [IsAuthenticated]
    queryset = PlayerGameData.objects.all()
    serializer_class = PlayerGameDataSerializer


class GuildGameDataSchemaViewSet(viewsets.ModelViewSet):
    permission_classes = [IsAuthenticated]
    queryset = GuildGameDataSchema.objects.all()
    serializer_class = GuildGameDataSchemaSerializer


class PlayerGuildViewSet(viewsets.ModelViewSet):
    permission_classes = [IsAuthenticated]
    queryset = PlayerGuild.objects.all()
    serializer_class = PlayerGuildSerializer


class WowClassesViewSet(viewsets.ModelViewSet):
    permission_classes = [IsAuthenticated]
    queryset = WowClasses.objects.all()
    serializer_class = WowClassesSerializer
